import commpy as comm
import numpy as np
from numpy.random import RandomState

class core:

    data, data1, data2, data3, data4, data5, data6, data7, data8, data9 = (0,0,0,0,0,0,0,0,0,0)
    data_sync, data_sync1, data_sync2, data_sync3, data_sync4, data_sync5, data_sync6, data_sync7, data_sync8 = (0,0,0,0,0,0,0,0,0)
    fs = 1e6
    params = {}
    
    def __init__(self, freq_shift = 0, timing_shift = 0, mod_type = 'PSK', mod_order = 2, snr_db = 100, coarce_enable = True, fine_enable = True, filter_beta = 0.35, timing_sync_coeff = 0.3, costas_alpha = 0.132, costas_beta = 0.00932, over_sample = 8, num_symbols = 125, rs_seed = 123456):
        self.params['freq_shift']         = freq_shift
        self.params['timing_shift']       = timing_shift
        self.params['mod_type']           = mod_type
        self.params['mod_order']          = mod_order
        self.params['over_sample']        = over_sample
        self.params['num_symbols']        = num_symbols
        self.params['filter_beta']        = filter_beta
        self.params['timing_sync_coeff']  = timing_sync_coeff
        self.params['costas_alpha']       = costas_alpha
        self.params['costas_beta']        = costas_beta
        self.params['rs_seed']            = rs_seed
        self.params['coarce_enable']      = coarce_enable
        self.params['snr_db']             = snr_db

            
    def create_source(self, rs):
        self.data = rs.randint(0, 2, self.params['num_symbols'] * int(np.log2(self.params['mod_order'])))
        self.data_sync = rs.randint(0, 2, self.params['num_symbols'])

    def modulate_data(self):
        if (self.params['mod_type'] == 'PSK') : self.data_modem = comm.PSKModem(self.params['mod_order'])
        elif (self.params['mod_type'] == 'QAM') : self.data_modem = comm.QAMModem(self.params['mod_order'])
        else: self.data_modem = None
        self.sync_modem = comm.PSKModem(2)
        self.data1 = self.data_modem.modulate(self.data)
        self.data_sync1 = self.sync_modem.modulate(self.data_sync)

    def upsample(self):
        self.data2 = np.zeros(self.params['over_sample']*(len(self.data1)-1)+1,dtype = np.complex64) 
        self.data2[::self.params['over_sample']] = self.data1
        self.data_sync2 = np.zeros(self.params['over_sample']*(len(self.data_sync1)-1)+1,dtype = np.complex64) 
        self.data_sync2[::self.params['over_sample']] = self.data_sync1

    def filter_data(self):
        Ts = self.params['over_sample']
        t = np.arange(-51, 52) # remember it's not inclusive of final number
        self.f = t/Ts
        # self.h = np.sinc(t/Ts) * np.cos(np.pi*(self.params['filter_beta']+0.00000000001)*t/Ts) / (1 - (2*(self.params['filter_beta']+0.00000000001)*t/Ts)**2)
        self.h = np.sinc(t/Ts) * np.cos(np.pi*(self.params['filter_beta'])*t/Ts) / (1 - (2*(self.params['filter_beta'])*t/Ts)**2)
        self.data3 = np.convolve(self.data2, self.h)
        self.data_sync3 = np.convolve(self.data_sync2, self.h)
        # return data_shaped,sync_data_shaped

    def add_awgn(self):
        self.data3_awgn = comm.awgn(self.data3, self.params['snr_db'])
        self.data_sync3_awgn = comm.awgn(self.data_sync3, self.params['snr_db'])
        
    def add_frac_delay(self):
        self.add_awgn()
        # Create and apply fractional delay filter
        N = 21 # number of taps
        n = np.arange(-N//2, N//2) # ...-3,-2,-1,0,1,2,3...
        h = np.sinc(n - self.params['timing_shift']) # calc filter taps
        h *= np.hamming(N) # window the filter to make sure it decays to 0 on both sides
        h /= np.sum(h) # normalize to get unity gain, we don't want to change the amplitude/power
        self.data4 = np.convolve(self.data3_awgn, h) # apply filter'
        self.data_sync4 = np.convolve(self.data_sync3_awgn, h) # apply filter

    def add_freq_shift(self):
        # apply a freq offset
        Ts = 1/self.fs # calc sample period
        t = np.arange(0, Ts*len(self.data4), Ts) # create time vector
        self.data5 = self.data4 * np.exp(1j*2*np.pi*self.params['freq_shift'] *t) # perform freq shift
        self.data_sync5 = self.data_sync4 * np.exp(1j*2*np.pi*self.params['freq_shift'] *t) # perform freq shift

    def sync_freq_coarse(self):
        sync_data_squared = self.data_sync5**2
        psd = np.fft.fftshift(np.abs(np.fft.fft(sync_data_squared)))
        f = np.linspace(-self.fs/2.0, self.fs/2.0, len(psd))
        max_freq = f[np.argmax(psd)]
        Ts = 1/self.fs # calc sample period
        t = np.arange(0, Ts*len(self.data_sync5), Ts) # create time vector
        self.data_sync6 = self.data_sync5 * np.exp(-1j*2*np.pi*max_freq*t/2.0)
        self.data6 = self.data5 * np.exp(-1j*2*np.pi*max_freq*t/2.0)
        sync_data_out_squared = self.data_sync6**2
        psd2 = np.fft.fftshift(np.abs(np.fft.fft(sync_data_out_squared)))
        f2 = np.linspace(-self.fs/2.0, self.fs/2.0, len(psd2))
        self.coarse_log = (f, psd, f2, psd2)

    def sync_symbol(self):
        mu = 0 # initial estimate of phase of sample
        out = np.zeros(len(self.data_sync6) + 10, dtype=complex)
        out_s = np.zeros(len(self.data6 ) + 10, dtype=complex)
        out_rail = np.zeros(len(self.data_sync6) + 10, dtype=complex) # stores values, each iteration we need the previous 2 values plus current value
        i_in = 0 # input samples index
        i_out = 2 # output index (let first two outputs be 0)
        while i_out < len(self.data_sync6) and i_in < len(self.data_sync6):
            out[i_out] = self.data_sync6[i_in + int(mu)] # grab what we think is the "best" sample
            out_s[i_out] = self.data6 [i_in + int(mu)] # grab what we think is the "best" sample
            out_rail[i_out] = int(np.real(out[i_out]) > 0) + 1j*int(np.imag(out[i_out]) > 0)
            x = (out_rail[i_out] - out_rail[i_out-2]) * np.conj(out[i_out-1])
            y = (out[i_out] - out[i_out-2]) * np.conj(out_rail[i_out-1])
            mm_val = np.real(y - x)
            mu += 8 + self.params['timing_sync_coeff']*mm_val
            i_in += int(np.floor(mu)) # round down to nearest int since we are using it as an index
            mu = mu - np.floor(mu) # remove the integer part of mu
            i_out += 1 # increment output index
        self.data_sync7 = out[2:i_out] # remove the first two, and anything after i_out (that was never filled out)
        self.data7  = out_s[2:i_out] 
        
    def sync_freq_fine(self):
        N = len(self.data_sync7)
        phase = 0
        freq = 0
        # These next two params is what to adjust, to make the feedback loop faster or slower (which impacts stability)
        self.data_sync8 = np.zeros(N, dtype=complex)
        self.data8 = np.zeros(N, dtype=complex)
        self.fine_log = []
        for i in range(N):
            self.data_sync8[i] = self.data_sync7[i] * np.exp(-1j*phase) # adjust the input sample by the inverse of the estimated phase offset
            self.data8[i] = self.data7[i] * np.exp(-1j*phase) # adjust the input sample by the inverse of the estimated phase offset
            error = np.real(self.data_sync8[i]) * np.imag(self.data_sync8[i]) # This is the error formula for 2nd order Costas Loop (e.g. for BPSK)

            # Advance the loop (recalc phase and freq offset)
            freq += (self.params['costas_beta'] * error)
            self.fine_log.append(freq / 50.0 * self.fs)
            phase += freq + (self.params['costas_alpha'] * error)

            # Optional: Adjust phase so its always between 0 and 2pi, recall that phase wraps around every 2pi
            while phase >= 2*np.pi:
                phase -= 2*np.pi
            while phase < 0:
                phase += 2*np.pi

    def demodulate(self):
        self.data9 = self.data_modem.demodulate(self.data8[8:], 'hard')
        self.compared = self.data9[0:len(self.data)] == self.data
        good =np.count_nonzero(self.compared == True)
        bad = np.count_nonzero(self.compared == False)
        ber = bad/(good + bad)
        self.ber_message = 'bad:' + str(bad) + '|' + 'good:'+ str(good) + ' | ber is:' + str(ber)

    def run(self):
        rs = RandomState(self.params['rs_seed'] )
        self.create_source(rs)
        self.modulate_data()
        self.upsample()
        self.filter_data()
        self.add_frac_delay()
        self.add_freq_shift()
        if(self.params['coarce_enable'] ):
            self.sync_freq_coarse()
        else:
            self.data6 = self.data5
            self.data_sync6 = self.data_sync5
            self.coarse_log = ([0], [0], [0], [0])
        self.sync_symbol()
        self.sync_freq_fine()
        self.demodulate()
    
    def check_for_update(self, new_params): 
        up_to_date = True
        for key in new_params.keys():
            if new_params[key] != self.params[key]: up_to_date = False
        return up_to_date
    
    def update(self, new_params):
        if (not self.check_for_update(new_params)): 
            for key in new_params.keys(): self.params[key] = new_params[key]
            self.run()




import numpy as np
from matplotlib import pyplot as plt
from core import core
# %matplotlib inline
# Prevents the pop-up graphs in a separate window.
get_ipython().run_line_magic('matplotlib', 'inline')

from ipywidgets import Tab 
from ipywidgets import fixed, Layout
from ipywidgets import widgets
class GUI:
    modulation_to_param = {
    'BPSK'      : ('PSK', 2),
    'QPSK'      : ('PSK', 4),
    'PSK-16'    : ('PSK', 16),
    'PSK-32'    : ('PSK', 32),
    'PSK-64'    : ('PSK', 64),
    'PSK-128'   : ('PSK', 128),
    'PSK-256'   : ('PSK', 256),
    'PSK-512'   : ('PSK', 512),
    'QAM-4'     : ('QAM', 4),
    'QAM-16'    : ('QAM', 16),
    'QAM-64'    : ('QAM', 64),
    'QAM-128'   : ('QAM', 256),
    }
    
    def __init__(self):
        self.core = core()
        self.core.run()

    def sir_interactive_func(self, freq_shift, timing_shift, mod , filter_beta, timing_sync_coeff, costas_alpha, costas_beta, coarce_enable, snr_db,  show_filtered_psd, plot_domain = 'time', plot_type = 'Source'):
        mod_type, mod_order = self.modulation_to_param[mod]
        params = {}
        params['freq_shift']         = freq_shift
        params['timing_shift']       = timing_shift
        params['mod_type']           = mod_type
        params['mod_order']          = mod_order
        params['filter_beta']        = filter_beta
        params['timing_sync_coeff']  = timing_sync_coeff
        params['costas_alpha']       = costas_alpha
        params['costas_beta']        = costas_beta
        params['coarce_enable']      = coarce_enable
        params['snr_db']             = snr_db
        self.core.update(params)

        if (plot_domain == 'time'):
            if plot_type == 'Source':
                f, axs = plt.subplots(2,1,figsize=(30,10))
                axs[0].set_title('Сгенерированные биты')
                axs[0].plot(range( self.core.data.shape[0]), self.core.data, '-*g' )
                axs[0].grid()
                axs[1].set_title('Модулированный сигнал')
                axs[1].plot(range( self.core.data1.real.shape[0]),  self.core.data1.real,'-*g' ,  self.core.data1.imag, '-*b' )
                axs[1].legend(['I', 'Q'])
                axs[1].grid()  
            
            elif plot_type == 'Filtered':
                if(show_filtered_psd):
                    f, axs = plt.subplots(2,1,figsize=(30,10))
                    axs[0].set_title('PSD сигнала до и  после формирующего фильтра')
                    axs[0].psd(self.core.data3)
                    axs[0].psd(self.core.data1)
                    axs[0].grid()
                else:
                    f, axs = plt.subplots(2,1,figsize=(30,10))
                    axs[0].set_title('Передискретизированный сигнал')
                    axs[0].plot(range(self.core.data2.real.shape[0]), self.core.data2.real, '-*g' )
                    axs[0].grid()
                axs[1].set_title('Сигнал после формирующего фильтра')
                axs[1].plot(range(self.core.data3.real.shape[0]), self.core.data3.real, '-*g' )
                axs[1].plot(np.arange(self.core.data1.real.shape[0])*self.core.params['over_sample']+51, self.core.data1.real, '*b' )   
                axs[1].legend(['Сигнал на выходе фильтра', 'Исходные отсчеты (с добавленной задержкой фильтров)'])
                axs[1].grid()  

            elif plot_type == 'Imparameters':
                f, axs = plt.subplots(2,1,figsize=(30,10))
                axs[0].set_title('Сигнал с фракционной задержкой') 
                axs[0].plot(np.arange(self.core.data3.real.shape[0])+11, self.core.data3.real, '-*r' )  
                axs[0].plot(range(self.core.data4.real.shape[0]), self.core.data4.real, '-*g' )
                axs[0].legend(['I сигнал до сдвига', 'I после сдвига'])
                axs[0].grid()
                axs[1].set_title('Сигнал со сдвигом по частоте') 
                axs[1].plot(range(self.core.data5.real.shape[0]), self.core.data5.real, '-*g', self.core.data5.imag, '-*b' )
                axs[1].legend(['I', 'Q'])
                axs[1].grid()  

            elif plot_type == 'Coarce_Timing':
                f1, psd1, f2, psd2 = self.core.coarse_log
                max_freq1 = f1[np.argmax(psd1)]
                max_freq2 = f2[np.argmax(psd2)]
                f, axs = plt.subplots(2,1,figsize=(30,10))
                axs[0].set_title('Отклонение частоты при грубой синхронизации')
                axs[0].grid()   
                axs[0].plot(f1, psd1)
                axs[0].plot(f2, psd2)
                arrowprops=dict(arrowstyle="->",connectionstyle="angle,angleA=0,angleB=60")
                axs[0].annotate(np.around(max_freq1,5), xy=(np.around(max_freq1, 5), psd1[np.argmax(psd1)]*0.95))
                axs[0].annotate(np.around(max_freq2,5), xy=(np.around(max_freq2, 5), psd2[np.argmax(psd2)]))

                axs[1].set_title('Отсчеты сигнала после тактовой синхронизации (реальная часть)') 
                axs[1].plot(np.arange(self.core.data3.real.shape[0])+12, self.core.data3.real[0:], '-*g' )
                a = np.arange(self.core.data7.real.shape[0]) *self.core.params['over_sample']
                axs[1].plot(a, self.core.data7.real[0:], 'or' )
                axs[1].plot(np.arange(self.core.data2.real[self.core.data2.real != 0].shape[0])*self.core.params['over_sample']+64, self.core.data2.real[self.core.data2.real != 0], 'bx' )
                axs[1].legend(['Исходные отсчеты (пересемплированные))', 'Выбранные отсчеты ', 'Отсчеты до пересемплирования'])
                axs[1].grid()

            elif plot_type == 'Fine':
                f, axs = plt.subplots(2,1,figsize=(30,10))
                axs[0].set_title('Отсчеты сигнала после петли Костаса') 
                axs[0].plot(np.arange(self.core.data3.real.shape[0])+12, self.core.data3.real[0:], '-*g' )
                a = np.arange(self.core.data8.real.shape[0]) * self.core.params['over_sample']
                axs[0].plot(a, self.core.data8.real[0:], 'or' )
                axs[0].plot(np.arange(self.core.data2.real[self.core.data2.real != 0].shape[0])*self.core.params['over_sample']+64, self.core.data2.real[self.core.data2.real != 0], 'bx' )
                
                axs[0].grid()
                axs[0].legend(['Исходные отсчеты (пересемплированные))', 'Выбранные отсчеты ', 'Отсчеты до пересемплирования'])
                
                axs[1].set_title('Значение ошибки в петле Костаса') 
                axs[1].plot(self.core.fine_log, '-*g' )
                axs[1].grid()

            elif plot_type == 'Demodulated':
                f, axs = plt.subplots(2,1,figsize=(30,10), sharex=True)
                axs[0].set_title('Исходеные  биты') 
                axs[0].plot(np.arange(len(self.core.data)), self.core.data, '-*r' )           
                axs[0].grid()

                axs[1].set_title('Демодулированные биты') 
                axs[1].plot(np.arange(len(self.core.data)), self.core.data9[0:len(self.core.data)], '-*g' )
                axs[1].grid()
                self.ber.value = self.core.ber_message
        else:
            if plot_type == 'Source':
                f, axs = plt.subplots(1,1,figsize=(30,10))
                axs.set_title('Сигнальное созвездие модулирвоанного сигнала')
                axs.plot(self.core.data1.real, self.core.data1.imag, '*')
                axs.grid()
                axs.axis('equal')
            
            elif plot_type == 'Filtered':
                f, axs = plt.subplots(1,2,figsize=(30,10))
                axs[0].set_title('Сигнальное созвездие передискретизированного сигнала')
                axs[0].plot(self.core.data2.real, self.core.data2.imag, '*' )
                axs[0].grid()
                axs[1].set_title('Сигнальное созвездие сигнала после формирующего фильтра')
                axs[1].plot(self.core.data3.real, self.core.data3.imag, '*' )
                axs[1].grid()  
                axs[0].axis('equal')
                axs[1].axis('equal')

            elif plot_type == 'Imparameters':
                f, axs = plt.subplots(1,2,figsize=(30,10))
                axs[0].set_title('Сигнальное созвездие сигнала с фракционной задержкой') 
                axs[0].plot(self.core.data4.real, self.core.data4.imag, '*' )
                axs[0].grid()
                axs[1].set_title('Сигнальное созвездие сигнала со сдвигом по частоте') 
                axs[1].plot(self.core.data5.real, self.core.data5.imag, '*' )
                axs[1].grid()
                axs[0].axis('equal')
                axs[1].axis('equal')  

            elif plot_type == 'Coarce_Timing':
                f1, psd1, f2, psd2 = self.core.coarse_log
                max_freq1 = f1[np.argmax(psd1)]
                max_freq2 = f2[np.argmax(psd2)]
                f, axs = plt.subplots(1,2,figsize=(30,10))
                axs[0].set_title('Сигнальное созвездие сигнала после грубой синхронизации по частоте')
                axs[0].plot(self.core.data6.real[50:-50], self.core.data6.imag[50:-50], '*' )
                axs[0].grid()
                axs[0].axis('equal')
                axs[1].axis('equal')

                axs[1].set_title('Сигнальное созвездие сигнала после тактовой синхронизации') 
                axs[1].plot(self.core.data7.real[50:-50], self.core.data7.imag[50:-50], '*' )
                axs[1].grid()
                axs[1].axis('equal') 

            elif plot_type == 'Fine':
                f, axs = plt.subplots(1,2,figsize=(30,10))
                axs[0].set_title('Сигнальное созвездие сигнала после петли Костаса')
                axs[0].plot(self.core.data8.real[50:-50], self.core.data8.imag[50:-50], '*' )
                axs[0].grid()
                axs[0].axis('equal')
                          
                axs[1].set_title('Значение ошибки в петле Костаса') 
                axs[1].plot(self.core.fine_log, '-*g' )
                axs[1].grid()

    def show(self):
        self.rfa_button = widgets.Button(description='Запустить анимацию изменения сдвига частоты', layout =  widgets.Layout(width='auto'))
        self.rta_button = widgets.Button(description='Запустить анимацию изменения временного сдвига', layout =  widgets.Layout(width='auto'))
        self.coarce_checkbox = widgets.Checkbox(False, description='Выполнять грубую синхронизацию по частоте', layout =  widgets.Layout(width='auto'))
        self.show_filtered_psd_checkbox = widgets.Checkbox(False, description='Показывать PSD до и после фильтрации', layout =  widgets.Layout(width='auto'))

        self.filter_beta_slider = widgets.FloatSlider(min=0,max=0.8,step=0.001, description="Коэффициент beta фильтра", layout = Layout(width='95%'), style = {'description_width': 'initial'})
        self.timing_sync_coeff_slider = widgets.FloatSlider(min=0,max=1,step=0.01, description="Множитель тактовой синхронизации", layout = Layout(width='95%'), style = {'description_width': 'initial'})
        self.costas_alpha_slider = widgets.FloatSlider(min=0,max=1,step=0.001, description="Коэффициент alpha в петле Костаса", layout = Layout(width='95%'), style = {'description_width': 'initial'})
        self.costas_beta_slider = widgets.FloatSlider(min=0,max=1,step=0.001, description="Коэффициент beta в петле Костаса", layout = Layout(width='95%'), style = {'description_width': 'initial'})
        self.dnr_db_slider = widgets.FloatSlider(min=0,max=100,step=1, value=100, description="SNR в дБ", layout = Layout(width='95%'), style = {'description_width': 'initial'})

        self.freq_shift_slider = widgets.FloatSlider(min=0,max=13000,step=1, description="Сдвиг по чатоте", layout = Layout(width='95%'), style = {'description_width': 'initial'})
        self.timing_shift_slider = widgets.FloatSlider(min=0,max=9.9,step=0.1, description="Сдвиг по времени", layout = Layout(width='95%'), style = {'description_width': 'initial'})
        self.plot_domain_list = widgets.Dropdown(options=['time', 'constellations'],description='Вид отображения', style = {'description_width': 'initial'})
        self.mod_list = widgets.Dropdown(options=self.modulation_to_param.keys(),description='Тип модуляции', style = {'description_width': 'initial'})

        self.ber = widgets.Text(layout={'border': '1px solid black'})
        self.base_map = { 'freq_shift': self.freq_shift_slider, 'timing_shift': self.timing_shift_slider, 'plot_domain':self.plot_domain_list, 'mod': self.mod_list, 
                     'filter_beta' : self.filter_beta_slider, 'timing_sync_coeff': self.timing_sync_coeff_slider, 'costas_alpha' : self.costas_alpha_slider, 
                     'costas_beta' : self.costas_beta_slider, 'coarce_enable':self.coarce_checkbox, 'show_filtered_psd': self.show_filtered_psd_checkbox, 'snr_db':self.dnr_db_slider,
                     'plot_type': fixed('Source')  }

        source_ui = widgets.HBox([self.mod_list, self.plot_domain_list]) 
        source_out = widgets.interactive_output(self.sir_interactive_func, {**self.base_map, 'plot_type': fixed('Source')} )
        source = widgets.VBox(children=[source_ui, source_out])

        # modulated_ui = widgets.HBox([self.mod_list, self.plot_domain_list])
        # modulated_out = widgets.interactive_output(self.sir_interactive_func, {**self.base_map, 'plot_type': fixed('Modulated')} )
        # modulated = widgets.VBox(children=[modulated_ui, modulated_out])

        filtered_ui = widgets.VBox([self.filter_beta_slider, widgets.HBox([self.mod_list, self.plot_domain_list, self.show_filtered_psd_checkbox])])
        filtered_out = widgets.interactive_output(self.sir_interactive_func, {**self.base_map, 'plot_type': fixed('Filtered')} )
        filtered  = widgets.VBox(children=[filtered_ui, filtered_out])

        imparameters_ui = widgets.VBox([self.freq_shift_slider, self.timing_shift_slider, self.dnr_db_slider, widgets.HBox([self.mod_list, self.plot_domain_list, self.rfa_button, self.rta_button])])
        imparameters_out = widgets.interactive_output(self.sir_interactive_func, {**self.base_map, 'plot_type': fixed('Imparameters')} )
        imparameters = widgets.VBox(children=[imparameters_ui, imparameters_out])


        coarce_timing_ui = widgets.VBox([self.freq_shift_slider,self.timing_shift_slider,self.dnr_db_slider, self.timing_sync_coeff_slider, widgets.HBox([self.mod_list, self.plot_domain_list, self.coarce_checkbox]) ])
        coarce_timing_out = widgets.interactive_output(self.sir_interactive_func, {**self.base_map, 'plot_type': fixed('Coarce_Timing')})
        coarce_timing = widgets.VBox(children=[coarce_timing_ui, coarce_timing_out])

        fine_ui = widgets.VBox([self.freq_shift_slider, self.timing_shift_slider, self.dnr_db_slider, self.costas_alpha_slider, self.costas_beta_slider, widgets.HBox([self.mod_list, self.plot_domain_list])])
        fine_out = widgets.interactive_output(self.sir_interactive_func, {**self.base_map, 'plot_type': fixed('Fine')})
        fine = widgets.VBox(children=[fine_ui, fine_out])  

        demodulated_ui = widgets.VBox([self.freq_shift_slider, self.timing_shift_slider, self.dnr_db_slider, self.costas_alpha_slider, self.costas_beta_slider, widgets.HBox([self.mod_list, self.plot_domain_list,  self.ber])])
        # demodulated_ui = widgets.VBox([self.freq_shift_slider, self.timing_shift_slider, self.dnr_db_slider, self.costas_alpha_slider, self.costas_beta_slider, widgets.HBox([self.mod_list, self.plot_domain_list])])
        demodulated_out = widgets.interactive_output(self.sir_interactive_func, {**self.base_map, 'plot_type': fixed('Demodulated')})
        demodulated =  widgets.VBox(children=[demodulated_ui, demodulated_out,])  

        self.rfa_button.on_click(self.run_freq_animation)
        self.rta_button.on_click(self.run_timing_animation)

        sub_tab=[source,  filtered,imparameters,coarce_timing,fine,demodulated ]
        self.tab = Tab(sub_tab)
        self.tab.set_title(0,"Source/Modulated")
        self.tab.set_title(1,"Upsampled/Filtered")
        self.tab.set_title(2,"Imparameters")
        self.tab.set_title(3,"Coarce/Timing")
        self.tab.set_title(4,"Fine")
        self.tab.set_title(5,"Demodulated")
        display(self.tab)

    def run_freq_animation(self, a, steps = 10, sep_size = 10):
        for i in range(steps):
            self.freq_shift_slider.value += sep_size

    def run_timing_animation(self, a, steps = 10, sep_size = 0.1):
        for i in range(steps):
            self.timing_shift_slider.value += sep_size


from setuptools import setup, find_packages

setup_args = dict(
    name = 'justanunsuspiciousmodule',
    version = '1.0',
    description='Class with a module for lab',
    license='MIT',
    packages=find_packages(),
    author = 'Mekatto', 
    author_email = 'igor@mekatto.com',
    url = 'https://github.com/Igor-Sviridov/justanunsuspiciousmodule',
    download_url = 'https://pypi.org/project/justanunsuspiciousmodule/',
    python_requires='>3.6.0',
)

install_requires = [
          'ipywidgets',
          'numpy',
          'matplotlib',


]

if __name__ == '__main__':
    setup(**setup_args, install_requires=install_requires)
